/**
 * \file phoenixmod.c
 * \brief building blocks for a computer-based unlocking and optimizing
 *        program aimed at official Nspire calculators OS
 *        Derived from my own tiosmod.
 * Copyright (C) 2011 Lionel Debroux (lionel underscore debroux yahoo fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 (and only version 2) of the
 * License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, 5th Floor, Boston, MA 02110-1301, USA
 */

#include <os.h>

// The function called by main().
void PatchPhoenix(void);


// Read data at the given absolute address.
static inline uint8_t GetByte (uint32_t absaddr) {
    return *((uint8_t *)absaddr);
}

static inline uint16_t GetShort (uint32_t absaddr) {
    return *((uint16_t *)absaddr);
}

static inline uint32_t GetLong (uint32_t absaddr) {
    return *((uint32_t *)absaddr);
}

static void GetNBytes (uint8_t * buffer, uint32_t n, uint32_t absaddr) {
    uint32_t i;
    uint32_t ptr = absaddr;
    for (i = 0; i < n; i++) {
        buffer[i] = GetByte(ptr);
        ptr++;
    }
}


// Write data at the given absolute address.
static inline void PutByte (uint8_t byte_in, uint32_t absaddr) {
    *((uint8_t *)absaddr) = byte_in;
}

static inline void PutShort (uint16_t word_in, uint32_t absaddr) {
    *((uint16_t *)absaddr) = word_in;
}

static inline void PutLong (uint32_t long_in, uint32_t absaddr) {
    *((uint32_t *)absaddr) = long_in;
}

static void PutNBytes (uint8_t *buffer, uint32_t n, uint32_t absaddr) {
    uint32_t i;
    uint32_t ptr = absaddr;
    for (i = 0; i < n; i++) {
        PutByte(buffer[i], ptr);
        ptr++;
    }
}


// Search forward for values, return the absolute address of the first byte after the value.
static uint32_t SearchByte (uint8_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (value != GetByte(ptr)) {
        ptr++;
    }
    return ptr;
}

static uint32_t SearchShort (uint16_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (value != GetShort(ptr)) {
        ptr += 2;
    }
    return ptr;
}

static uint32_t SearchLong (uint32_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (value != GetLong(ptr)) {
        ptr += 4;
    }
    return ptr;
}


// Search backwards for values, return the absolute address of the value.
static uint32_t SearchBackwardsByte (uint8_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (GetByte(ptr) != value) {
        ptr -= 2;
    }
    ptr--;
    return ptr;
}

static uint32_t SearchBackwardsShort (uint16_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (GetShort(ptr) != value) {
        ptr -= 4;
    }
    ptr -= 2;
    return ptr;
}

static uint32_t SearchBackwardsLong (uint32_t value, uint32_t absaddr) {
    uint32_t ptr = absaddr;
    while (GetLong(ptr) != value) {
        ptr -= 6;
    }
    ptr -= 4;
    return ptr;
}


int main(void) {

    PatchPhoenix();

    return 0;
}
